//// -*- mode: Java; coding: utf-8 -*-
/******************************************************************************
** This data and information is proprietary to, and a valuable trade secret
** of, Basis Technology Corp.  It is given in confidence by Basis Technology
** and may only be used as permitted under the license agreement under which
** it has been distributed, and in no other way.
**
** Copyright (c) 2008 Basis Technology Corp. All rights reserved.
**
** The technical data and information provided herein are provided with
** `limited rights', and the computer software provided herein is provided
** with `restricted rights' as those terms are defined in DAR and ASPR
** 7-104.9(a).
******************************************************************************/
package com.basistech.rlp.extra;

import java.io.File;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.basistech.LoggingChannels;
import com.basistech.rlp.EnvironmentParameters;
import com.basistech.rlp.RLPEnvironment;
import com.basistech.rlp.RLPException;
import com.basistech.util.Pathnames;
/**
 * Creates and manages the one and only instance of RLPEnvironment.
 * 
 */
public class RLPEnvironmentHolder {
    private static RLPEnvironment env = null;
    private static Logger logger = LoggerFactory.getLogger(RLPEnvironmentHolder.class);
    private static final String BT_RLP_ENV_PROP = "bt.rlp.env";
    private static String btRoot;

    // Prevent this class from being instantiated.
    private RLPEnvironmentHolder(){}
    
    private static synchronized void initOnce() throws RLPException{
        if(env != null) return;
        try{
            EnvironmentParameters envparams = new EnvironmentParameters();
            btRoot = (new Pathnames()).getBTRootDirectory(); // Get the "bt.root" value.
            String envConfigPath = System.getProperty(BT_RLP_ENV_PROP, btRoot + "/rlp/etc/rlp-global.xml");
            if(!(new File(envConfigPath)).exists()){
                throw new RuntimeException(envConfigPath + " does not exist.  Are bt.root and " + BT_RLP_ENV_PROP + "(optional) set properly?");
            }
            envparams.setEnvironmentDefinition(new File(envConfigPath));
            
            envparams.setLogLevel("all");
            envparams.setLogCallback(new RLPEnvironment.LogCallback() {
              public void message(int channel, String message) {
                switch(channel){
                case LoggingChannels.WARNING_CHANNEL: // Warning
                        logger.warn(message); break;
                case LoggingChannels.INFO_CHANNEL: // Info
                        logger.info(message); break;
                case LoggingChannels.ERROR_CHANNEL: // Error
//              case LoggingChannels.MEMORY_CHANNEL: // Internal
//              case LoggingChannels.PERFORMANCE_CHANNEL: // Internal
                default: // To be on the safer side, we map any other values as fatal so that they have a better chance to be logged.
                        logger.error(message);
                }
              }
            }
            );
                                   
            env = new RLPEnvironment(envparams);
            env.initialize();
            return;
        }catch(Exception ex){
            throw new RLPException(ex, "Error creating an RLPEnvironment object.");
        }
    }

    /**
     * Returns the one and only RLPEnvironment object.
     * On the first call to any method of this class, RLPEnvironment object will be initialized with:
     * <ul>
     * <li>bt.root system property that should point to the RLP installation directory.
     * This is the directory that has the "rlp" subdirectory.</li>
     * <li>bt.rlp.env system property that points to an XML file defining RLP Environment.
     * This is optional.
     * If omitted, <i>bt.root</i>/rlp/etc/rlp-global.xml will be assumed.</li>
     * </ul>
     * It binds RLP log messages to <a href="http://www.slf4j.org">SL4J (Simple Logging Facade for Java)</a>.
     * The classpath needs to include the SL4J API JAR (e.g. slf4j-api-1.5.2.jar) and one of the adapter JARs (slf4j-jdk14-1.5.2.jar) at run time.
     * @return the RLPEnvironment object
     */
    public static RLPEnvironment getEnvironment()
    throws RLPException
    {
        initOnce();
        return env;
    }
    
    /**
     * A convenient method to return the product root directory, used to initialize the RLPEnvironment object.
     * @return the Basis Technology product installation root directory
     */
    public static String getBTRootDirectory()
    throws RLPException
    {
        initOnce();
        return btRoot;
    }
}
