//// -*- mode: Java; coding: utf-8 -*-
/******************************************************************************
** This data and information is proprietary to, and a valuable trade secret
** of, Basis Technology Corp.  It is given in confidence by Basis Technology
** and may only be used as permitted under the license agreement under which
** it has been distributed, and in no other way.
**
** Copyright (c) 2007 Basis Technology Corp. All rights reserved.
**
** The technical data and information provided herein are provided with
** `limited rights', and the computer software provided herein is provided
** with `restricted rights' as those terms are defined in DAR and ASPR
** 7-104.9(a).
******************************************************************************/
package com.basistech.rlp.lucene;

import java.io.Reader;
import java.util.EnumSet;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.LowerCaseFilter;
import org.apache.lucene.analysis.TokenStream;

import com.basistech.util.LanguageCode;

import com.basistech.rlp.lucene.RLPTokenizer.PostType;

/**
 * A generic (language-neutral) Analyzer that uses RLPTokenizer.
 * <br/>
 * This Analyzer uses {@link RLPTokenizer}, {@link org.apache.lucene.analysis.LowerCaseFilter},
 * and {@link RLPPOSFilter} (only if POS generation is turned on and the allowed POS tag list is provided).
 */
public class RLPAnalyzer extends Analyzer {

    /**
     * Gets the RLP Context Definition string that is assumed when a constructor that doesn't take one is used.
     * The Context Definition contains RLI (for language detection), RCLU (for Unicode NKFC normalization), and 
     * all base linguistics language processors (LPs).
     * For these LPs to work, make sure your RLP license covers these LPs for the languages you use.
     * @return The default context definition string.
     */
    public static String getDefaultContextDefinition()
    {
        return DEFAULT_CONTEXT_DEF_STRING;
    }
    private static final String DEFAULT_CONTEXT_DEF_STRING
    = "<?xml version=\"1.0\"  standalone=\"no\"?>"
    + "<!DOCTYPE contextconfig SYSTEM \"http://www.basistech.com/dtds/2003/contextconfig.dtd\">"
    + "<contextconfig>"
    + " <properties>"
    + "  <property name=\"com.basistech.rclu.FormKCNormalization\" value=\"true\"/>"
    + " </properties>"
    + " <languageprocessors>"
    + "  <languageprocessor>RLI</languageprocessor>"
    + "  <languageprocessor>RCLU</languageprocessor>"
    + "  <languageprocessor>BL1</languageprocessor>"
    + "  <languageprocessor>JLA</languageprocessor>"
    + "  <languageprocessor>CLA</languageprocessor>"
    + "  <languageprocessor>KLA</languageprocessor>"
    + "  <languageprocessor>Tokenizer</languageprocessor>"
    + "  <languageprocessor>SentenceBoundaryDetector</languageprocessor>"
    + "  <languageprocessor>ARBL</languageprocessor>"
    + "  <languageprocessor>FABL</languageprocessor>"
    + "  <languageprocessor>URBL</languageprocessor>"
    + " </languageprocessors>"
    + "</contextconfig>";
    
    /**
     * Gets the set of post types that is assumed when a constructor that doesn't take a post types argument is used.
     * It includes every post type except POS (part of speech).
     * @return The default set of post-types.
     */
    public static EnumSet<PostType> getDefaultPostTypes()
    {
        return ALL_POST_TYPES_BUT_POS.clone();
    }
    private final static EnumSet<PostType> ALL_POST_TYPES_BUT_POS = EnumSet.complementOf(EnumSet.of(PostType.POS));
    
    private LanguageCode lang;
    private String rlpContextDef;
    private RLPTokenizer rlpTzr;
    private EnumSet<PostType> postTypes;
    private String[] allowedPOSTags;
        
    /**
     * The most flexible constructor takes 4 arguments.
     * @param lang The language of the text. If it is {@link LanguageCode#UNKNOWN}, 
     * then RLI will be used to auto-detect the language. To use RLI, rlpContextDef must include RLI, 
     * and the RLI feature must be licensed.
     * @param rlpContextDef Either the file path or the definition itself in XML form.
     * @param postTypes Essentially a bit vector that tells what RLP result types should generate tokens. 
     * Specify EnumSet.allOf(PostType.class) to enable all possible token types with part-of-speech tags.
     * @param allowedPOSTags List of part-of-speech tags that should not be filtered out; null means no 
     * POS filtering. Note part-of-speech tags are language dependent. This argument will be ignored and 
     * no POS filter will be used if postTypes does not include POS.
     */
    public RLPAnalyzer(LanguageCode lang, String rlpContextDef, EnumSet<PostType> postTypes, String allowedPOSTags[] )
    {
        this.lang = lang;
        this.rlpContextDef = rlpContextDef;
        this.postTypes = postTypes;
        this.allowedPOSTags = allowedPOSTags;
    }

    /**
     * When this constructor is used, no POS filter will be used.
     * Make sure that postTypes does not contains PostType.POS for optimal performance.
     * @param lang The language of the text. If it is {@link LanguageCode#UNKNOWN}, then RLI will be used to auto-detect the language.  To use RLI, rlpContextDef must include RLI, and the RLI feature must be licensed.
     * @param rlpContextDef Either the file path or the definition itself in XML form.
     * @param postTypes Essentially a bit vector that tells what RLP result types should generate tokens. Specify EnumSet.allOf(PostType.class) to enable every possible token types with part-of-speech tags.
     */
    public RLPAnalyzer(LanguageCode lang, String rlpContextDef, EnumSet<PostType> postTypes)
    {
        this(lang, rlpContextDef, postTypes, null);
    }

    /**
     * Same as the three-parameter constructor except that Tokens derived from all the available result 
     * types will be generated without POS tags in their payload.
     * @param lang The language of the text. If it is {@link LanguageCode#UNKNOWN}, then RLI will 
     * be used to auto-detect the language. To use RLI, rlpContextDef must include RLI, and the 
     * RLI feature must be licensed.
     * @param rlpContextDef Either the file path or the definition itself in XML form.
     */
    public RLPAnalyzer(LanguageCode lang, String rlpContextDef)
    {
        this(lang, rlpContextDef, ALL_POST_TYPES_BUT_POS, null);
    }

    /**
     * Same as the two-parameter constructor except that an RLP Context that includes RCLU and the base linguistic 
     * LPs for all supported languages will be used. The Context Def will also include RLI if lang == null. 
     * RCLU will be used for Unicode NKFC normalization. 
     * @param lang The language of the text. If it is {@link LanguageCode#UNKNOWN}, then RLI will be used 
     * to auto-detect the language.  To use RLI, rlpContextDef must include RLI, and the RLI feature 
     * must be licensed.
     */
    public RLPAnalyzer(LanguageCode lang)
    {
        this(lang, DEFAULT_CONTEXT_DEF_STRING, ALL_POST_TYPES_BUT_POS, null);
    }

    /**
     * This is equivalent of RLPAnalyzer({@link LanguageCode#UNKNOWN}). RLI feature must be licensed. 
     */
    public RLPAnalyzer()
    {
        this(LanguageCode.UNKNOWN, DEFAULT_CONTEXT_DEF_STRING, ALL_POST_TYPES_BUT_POS, null);
    }
    
    /**
     * An implementation of {@link Analyzer#tokenStream(String, Reader)}.
     * @param fieldName Not used.
     * @param reader Input reader.
     * @return The TokenStream generated by RLPTokenizer and LowerCaseFilter. 
     * When the allowedPOSTags arg is specified and the postTypes argument includes POS, RLPPOSFilter 
     * is also used.
     */
    @Override
    public TokenStream tokenStream(String fieldName, Reader reader)
    {
        rlpTzr = new RLPTokenizer(reader, lang, rlpContextDef, postTypes);
        
        if(allowedPOSTags == null || allowedPOSTags.length == 0 || !postTypes.contains(PostType.POS)){// No RLPPOSFilter
            return new LowerCaseFilter(rlpTzr);
        }else{
            return new LowerCaseFilter(
                    new RLPPOSFilter(rlpTzr, allowedPOSTags));
        }
    }

    /**
     * Returns the language detected by RLI, if it is enabled. Otherwise, it returns the language set by the constructor.
     * <br/>
     * This method only makes sense when used from RLPAnalyzer, not from a subclass specialized for a language.
     * @return The detected language.
     */
    public LanguageCode getDetectedLanguage(){
        return rlpTzr.getDetectedLanguage();
    }
    
}
