//// -*- mode: Java; coding: utf-8 -*-
/******************************************************************************
** This data and information is proprietary to, and a valuable trade secret
** of, Basis Technology Corp.  It is given in confidence by Basis Technology
** and may only be used as permitted under the license agreement under which
** it has been distributed, and in no other way.
**
** Copyright (c) 2007 Basis Technology Corp. All rights reserved.
**
** The technical data and information provided herein are provided with
** `limited rights', and the computer software provided herein is provided
** with `restricted rights' as those terms are defined in DAR and ASPR
** 7-104.9(a).
******************************************************************************/
package com.basistech.rlp.lucene;

import java.io.IOException;
import java.util.EnumSet;


import lia.analysis.AnalyzerUtils;


import org.apache.lucene.analysis.Analyzer;

import com.basistech.rlp.lucene.RLPTokenizer.PostType;
import com.basistech.util.LanguageCode;

/**
 * An Analyzer for English that uses RLP.
 * To use this analyzer, you must have a valid RLP license that enables the 
 * Base Linguistics language processor for European languages (BL1 LP).
 * <br/>
 * This Analyzer uses {@link RLPTokenizer}, {@link org.apache.lucene.analysis.LowerCaseFilter},
 * and {@link RLPPOSFilter} (only if POS generation is turned on and the allowed POS tag list is provided).
 * <br/> 
 * Note: Although this is currently implemented as a subclass of {@link RLPAnalyzer},
 * this is regarded as an implementation detail, and it may change in the future.
 * The eventual contract is that it is a subclass of Lucene {@link org.apache.lucene.analysis.Analyzer}.
 */
public class RLPEnAnalyzer extends RLPAnalyzer 
{
    /**
     * Gets the default context definition, which only contains the BL1 LP.
     * @return The context definition string in XML that the default constructor uses.
     */
    public static String getDefaultContextDefinition()
    {
        return DEFAULT_CONTEXT_DEF;
    }    
    private static final String DEFAULT_CONTEXT_DEF = "<?xml version=\"1.0\"  standalone=\"no\"?>" +
    "<!DOCTYPE contextconfig SYSTEM \"http://www.basistech.com/dtds/2003/contextconfig.dtd\">" +
    "<contextconfig>" +
    " <languageprocessors>" +
    "  <languageprocessor>BL1</languageprocessor>" +  
    " </languageprocessors>" +
    "</contextconfig>";



    /**
     * Gets the array of part-of-speech (POS) tags that is assumed when constructor without such argument is used.
     * The list is made of the POS tags of word tokens.
     * Fine-tuning this list is recommended for optimal search results.
     * @return The default array of allowed part-of-speech tags that the default constructor uses.
     */
    public static String[] getDefaultAllowedPOSTags()
    {
        return DEFAULT_ALLOWED_POS_TAGS.clone();
    }
    private static String DEFAULT_ALLOWED_POS_TAGS[]=new String[]{
//      (basic) adjective: [a] blue [book], [he is] big 
        "ADJ",
//         comparative adjective: [he is] bigger, [a] better [question]
        "ADJCMP", 
//         adjectival ing-form: [the] working [men]
        "ADJING",
//         adjectival past participle: [a] locked [door]
        "ADJPAP",
//         pronoun (with determiner) or adjective: [the] same; [the] other [way]
        "ADJPRON",
//         superlative adjective:[he is the] biggest; [the] best [cake]
        "ADJSUP", 
//         (basic) adverb: today, quickly
        "ADV",
//         comparative adverb: sooner
        "ADVCMP",
//         superlative adverb: soonest
        "ADVSUP",
//         cardinal (except one): two, 123, IV
        "CARD",
//         cardinal one: [at] one [time] ; one [dollar]
        "CARDONE",
//         comma (",")
//        "CM",
//         coordination adverbs "either", "neither": either [by law or by force];[he didn't come] either
//        "COADV",
//         coordinating conjunction: and, or
//        "COORD",
//         subordinating conjunction: because, while
//        "COSUB",
//         conjunction "than": [bigger] than
//        "COTHAN",
//         determiner: the [house], a [house], this [house], my [house]
//        "DET",
//         relative determiner "whose": [the man] whose [hat ...]
//        "DETREL",
//         infinitive marker "to": [he wants] to [go]
//        "INFTO",
//         interjection: oh!
//        "ITJ",
//         measure abbreviation: [50] m. [wide], yd
        "MEAS",
//         currency plus cardinal: $1,000
        "MONEY",
//         negation "not": [he will] not [come in]
//        "NOT",
//         common noun: house
        "NOUN",
//         nominal ing-form: [the] singing [was pleasant], [the] raising [of the flag]
        "NOUNING",
//         ordinal: 3rd, second
        "ORD",
//         past participle (in subclause): [while] seated[, heinstructed the students]; [the car] sold [on Monday]
        "PARTPAST",
//         present participle (in subclause), gerund: [while] doing [it];[they began] designing [the ship];having [said this ...]
        "PARTPRES",
//         possessive suffix 's: [Peter] 's ; [houses] '
//        "POSS",
//         pre-determiner "such": such [a way]
//        "PREDET",
//         preposition: in [the house], on [the table]
//        "PREP",
//         preposition or adverbial "as": as [big] as
//        "PREPADVAS",
//         (non-personal) pronoun everybody, this [is ] mine
//        "PRON",
//         pronoun one: one [of them]; [the green] one
//        "PRONONE",
//         personal pronoun: I, me, we, you
//        "PRONPERS",
//         reflexive pronoun: myself, ...
//        "PRONREFL",
//         relative pronoun "who", "whom", "whose", "which", "that": [the man] who [wrote that book], [the ship] that capsized
//        "PRONREL",
//         proper namea: Peter, [Mr.] Brown
        "PROP",
//         punctuation (other than SENT and CM): "
//        "PUNCT",
//         quantifier "all", "any", "both", "double", "each", "enough", "every", (a) "few", "half", "many", "some":
//         many [people]; half [the price]; all [your children]; enough [time]; any [of these]
//        "QUANT",
//         quantifier or adverb "much", "little": much [better] , [he cares] little
//        "QUANTADV",
//         quantifier or comparative adverb "more", "less": more [people], less [expensive]
//        "QUANTCMP",
//         quantifier or superlative adverb: most, least
//        "QUANTSUP",
//         sentence final punctuation: . ! ? :
//        "SENT",
//         title Mr., Dr.
        "TIT",
//         auxiliary (modal): [he] will [run], [I] won't [come]
//        "VAUX",
//         infinitive or imperative of be:[he will] be [running]; be [quiet!]
//        "VBI",
//         past participle of be: [he has] been [there]
//        "VBPAP",
//         past tense of be: [he] was [running], [he] was [here]
//        "VBPAST",
//         present tense of be: [he] is [running], [he] is [old]
//        "VBPRES",
//         ing-form of be: [it is] being [sponsored]
//        "VBPROG",
//         infinitive of do: [He will] do [it]
//        "VDI",
//         past participle of do: [he has] done [it]
//        "VDPAP",
//         past tense of do:[we] did [it], [he] didn't [come]
//        "VDPAST",
//         present tense of do:[We] do [it], [he] doesn't [go]
//        "VDPRES",
//         ing-form of do: [He is] doing [it]
//        "VDPROG",
//         infinitive or imperative of have: [he would] have [come]; have [a look!]
        "VHI",
//         past participle of have: [he has] had [a cold]
        "VHPAP",
//         past tense of have: [he] had [seen]
        "VHPAST",
//         present tense of have: [he] has [been watching]
        "VHPRES",
//         ing-form of have: [he is] having [a good time]
        "VHPROG",
//         verb infinitive or imperative:[he will] go, [he comes to] see; listen [!]
        "VI",
//         verb past participle: [he has] played, [it is] written
        "VPAP",
//         verb past tense: [I] went, [he] loved
        "VPAST",
//         verb present tense: [we] go, [she] loves
        "VPRES",
//         verb ing-form: [you are] going
        "VPROG",
//         verbal 's (short for "is" or "has"): [he] 's [coming]
        "VS"
//         interrogative adverb: when [did ...], where [did ...], why [did ...]
//        "WADV",
//         interrogative determiner: which [book], whose [hat]
//        "WDET",
//         interrogative pronoun: who [is], what [is]
//        "WPRON"
       };    

    /**
     * Gets the set of post types that is assumed when a constructor without such argument is used.
     * For English, this contains only STEM and POS.
     * WORD is turned off as it would just double the number of Tokens.
     * @return The default set of post types.
     */
    public static EnumSet<PostType> getDefaultPostTypes()
    {
        return DEFAULT_POST_TYPES.clone();
    }
    private static final EnumSet<PostType> DEFAULT_POST_TYPES = EnumSet.of(
            PostType.STEM, 
            PostType.POS);


    /**
     * This constructor does not use default values.
     * @param rlpContextDef Context definition that RLP uses to process text: an XML string or path to XML file.
     * @param postTypes RLP Result types for which the tokenizer will generate tokens.
     * @param allowedPOSTags POSTagFilter will accept tokens with these POS tags.
     * @see RLPAnalyzer#RLPAnalyzer(LanguageCode, String, EnumSet, String[])
     */
    public RLPEnAnalyzer(String rlpContextDef, EnumSet<PostType> postTypes, String allowedPOSTags[])
    {
        super(LanguageCode.ENGLISH, rlpContextDef, postTypes, allowedPOSTags);
    }

    /**
     * This constructor uses the part-of-speech filter with the default part-of-speech tag set.
     * @param rlpContextDef Context definition that RLP uses to process text: an XML string or path to XML file.
     * @param postTypes RLP Result types for which the tokenizer will generate tokens.
     * @see RLPAnalyzer#RLPAnalyzer(LanguageCode, String, EnumSet)
     */
    public RLPEnAnalyzer(String rlpContextDef, EnumSet<PostType> postTypes)
    {
        super(LanguageCode.ENGLISH, rlpContextDef, postTypes, DEFAULT_ALLOWED_POS_TAGS);
    }

    /**
     * This constructor uses default set of the post types, which are <ul>
     * <li>STEM (which is actually a lemma)</li>
     * <li>POS (part-of-speech in Token's payload field)</li>
     * </ul>
     * @param rlpContextDef Context definition that RLP uses to process text: an XML string or path to XML file.
     * @see RLPAnalyzer#RLPAnalyzer(LanguageCode, String)
     */
    public RLPEnAnalyzer(String rlpContextDef)
    {
        super(LanguageCode.ENGLISH, rlpContextDef, DEFAULT_POST_TYPES, DEFAULT_ALLOWED_POS_TAGS);        
    }

    /**
     * This default constructor uses a default RLP Context, which only includes BL1 LP, along with
     * the default set of post types, and the default POS tags for English processing.
     * @see RLPAnalyzer#RLPAnalyzer(LanguageCode)
     */
    public RLPEnAnalyzer()
    {
        super(LanguageCode.ENGLISH, DEFAULT_CONTEXT_DEF, DEFAULT_POST_TYPES, DEFAULT_ALLOWED_POS_TAGS);
    }

        
    /**
     * (Internal use only)
     * 
     * Tokenizes an English sentence and displays the results.
     * 
     * @param args The English sentence to process (arg[0]). If you do not include an arg, a 
     * default sentence is processed.
     */
    public static void main(String[] args) 
    {

        Analyzer anaLemmaOnly = new RLPEnAnalyzer(DEFAULT_CONTEXT_DEF, EnumSet.of(PostType.STEM));
        Analyzer anaNoLemma = new RLPEnAnalyzer(DEFAULT_CONTEXT_DEF, EnumSet.of(PostType.WORD));
        Analyzer anaLemmaSyn = new RLPEnAnalyzer(DEFAULT_CONTEXT_DEF, EnumSet.of(PostType.WORD, PostType.STEM));
        Analyzer anaLemmaOnlyWithPOS = new RLPEnAnalyzer(DEFAULT_CONTEXT_DEF, EnumSet.of(PostType.STEM, PostType.POS));

        
        String text = "Crown Prince Naruhito's four-year old daughter, Princess Aiko, attended the ceremony with her parents and 50 of her future classmates, and later met her teachers, Kyodo news agency reported.";
        if(args.length>=1){
            text = args[0];
        }
        try {
            System.out.println("Analyzing:"+text);
            AnalyzerUtils.displayTokensWithFullDetails(anaLemmaOnly, text);
            AnalyzerUtils.displayTokensWithFullDetails(anaNoLemma, text);
            AnalyzerUtils.displayTokensWithFullDetails(anaLemmaSyn, text);
            AnalyzerUtils.displayTokensWithFullDetails(anaLemmaOnlyWithPOS, text);  
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
