//// -*- mode: Java; coding: utf-8 -*-
/******************************************************************************
** This data and information is proprietary to, and a valuable trade secret
** of, Basis Technology Corp.  It is given in confidence by Basis Technology
** and may only be used as permitted under the license agreement under which
** it has been distributed, and in no other way.
**
** Copyright (c) 2007 Basis Technology Corp. All rights reserved.
**
** The technical data and information provided herein are provided with
** `limited rights', and the computer software provided herein is provided
** with `restricted rights' as those terms are defined in DAR and ASPR
** 7-104.9(a).
******************************************************************************/
package com.basistech.rlp.lucene;

import java.io.IOException;
import java.util.EnumSet;


import lia.analysis.AnalyzerUtils;


import org.apache.lucene.analysis.Analyzer;

import com.basistech.rlp.lucene.RLPTokenizer.PostType;
import com.basistech.util.LanguageCode;


/**
 * An Analyzer for Japanese that uses RLP.
 * To use this analyzer, you must have a valid RLP license that enables the JLA language processor.
 * The default RLP context definition also requires RCLU to be licensed. RCLU is used for 
 * Form KC Normalization and lowercasing. 
 * <br/>
 * This Analyzer uses {@link RLPTokenizer}, {@link org.apache.lucene.analysis.LowerCaseFilter},
 * and {@link RLPPOSFilter} (only if POS generation is turned on and the allowed POS tag list is provided.
 * <br/>
 * Note: Although this is currently implemented as a subclass of {@link RLPAnalyzer},
 * this is regarded as an implementation detail, and it may change in the future.
 * The eventual contract is that it is a subclass of Lucene {@link org.apache.lucene.analysis.Analyzer}.
 */

 public final class RLPJaAnalyzer extends RLPAnalyzer 
{	
     /**
      * Gets the context definition that is assumed when a constructor without such argument is used.
      * For the Japanese Analyzer, the context includes RCLU and JLA.
      * RCLU is used to normalize the input to Unicode Normalization Form KC, which would contains
      * katakana letters in its fullwidth form, Latin letters and symbols in the halfwidth form, 
      * and lowercase Latin letters. 
      * 
      * @return The context definition string in XML that the default constructor uses.
      */
     public static String getDefaultContextDefinition()
     {
         return DEFAULT_CONTEXT_DEF;
     }
     private static final String DEFAULT_CONTEXT_DEF = "<?xml version=\"1.0\"  standalone=\"no\"?>" +
     "<!DOCTYPE contextconfig SYSTEM \"http://www.basistech.com/dtds/2003/contextconfig.dtd\">" +
     "<contextconfig>" +
     " <properties>" +
     "  <property name=\"com.basistech.rclu.FormKCNormalization\" value=\"true\"/>" +
     "  <property name=\"com.basistech.rclu.ToLowercase\" value=\"true\"/>" +
     "  <property name=\"com.basistech.jla.deep_compound_decomposition\" value=\"true\"/>" +
     " </properties>" +
     " <languageprocessors>" +
     "  <languageprocessor>RCLU</languageprocessor>" +  
     "  <languageprocessor>JLA</languageprocessor>" +	   
     " </languageprocessors>" +
     "</contextconfig>";

     /**
      * Gets the array of part-of-speech (POS) tags that is assumed when constructor without such argument is used.
      * The list is made of these POS tags which basically represent all content words such as these:
      * <ul>
      * <li>Nouns(名詞) except pronouns(代名詞)</li>
      * <li>Verbs(動詞)</li>
      * <li>Adjectives(形容詞) and Adjectival nouns(形容動詞語幹) except この, いわゆる, etc.</li>
      * <li>Derivational suffixes(派生的接尾辞) such as (東京)都, (中央)区, (男)っぽい</li>
      * <li>Others(その他) and unknown(不明)</li>
      * </ul>
      * Fine-tuning this list is recommended for optimal search result.
      * @return The default array of allowed part-of-speech tags that the default constructor uses.
      */
     public static String[] getDefaultAllowedPOSTags()
     {
         return DEFAULT_ALLOWED_POS_TAGS.clone();
     }  
     private static String DEFAULT_ALLOWED_POS_TAGS[]=new String[]
                                                                 {
         "NC",   // Common Noun (普通名詞)
         "NP",   // Proper Noun (固有名詞)
         "NE",   // Affixal Prefix (接頭辞性名詞) : 明治
         "NN",   // Numeral (数詞)　: 三, 千, 二つ, 数(種類)
         "NU",   // Units (助数詞) : （一)冊, (二)本, (3)種類
         //      "NR",   // Pronoun (代名詞) : 誰, あれ, 私
         "V",    // Verb (動詞)
         "V1",   // Verb (動詞)
         "VN",   // Verb (動詞)
         "VS",   // Verb (動詞)
         "VX",   // Verb (動詞)
         "V5",   // Verb (動詞)
         //      "AA",   // Adnominal Adjective (連体詞) : この, いわゆる
         "AJ",   // Adjective (形容詞) : 赤い, 美しい
         "AN",   // Adjectival Noun (形容動詞語幹) : 元気, 堂々
         //      "FP",   // Adnominal Prefix (形容動詞活用語尾) : な, だった
         //      "FS",   //Suffix : な, だった, 中, 後     
         //      "HP",   //Honorific Prefix (待遇接頭辞) : お, ご, 御
         //      "HS",   //Honorific Suffix (待遇接尾辞) : くん, ちゃん
         //      "WP",   // Derivational Prefix (派生的接頭語) : ぶっ(倒れる)
         "WS",   // Derivational Suffix (派生的接尾辞) : (東京)都, (中央)区, (男)っぽい
         //      "D",    // Adverb (副詞) : さらに
         //      "I",    // Interjection (感動詞) : あら、さようなら
         //      "J",    // Conjunction (接続詞) : そして, しかし
         //      "PL",   // Particle (助詞) : が, を, から
         "O",    // Word written in non-Japanese script such as: NHK, VIP
         "UNKNOWN" // (不明)
                                                                 };    

     /**
      * Gets the set of post types that is assumed when a constructor without such argument is used.
      * For Japanese, this contains STEM, COMP (compound decomposition), and POS.
      * WORD is turned off as it would just double the number of Tokens.
      * READING is not included, as it is not likely to be used in search.
      * M1NORM is not included, as it is an optional feature.
      * @return The default set of post types.
      */
     public static EnumSet<PostType> getDefaultPostTypes()
     {
         return DEFAULT_POST_TYPES.clone();
     }
     private static final EnumSet<PostType> DEFAULT_POST_TYPES = EnumSet.of(
             PostType.STEM, 
             PostType.COMP,
             PostType.POS);
     /**
      * This constructor does not use default values.
      * @param rlpContextDef Context definition that RLP uses to process text: an XML string or path to XML file.
      * @param postTypes RLP Result types for which the tokenizer will generate tokens.
      * @param allowedPOSTags POSTagFilter will accept tokens with these POS tags.
      * @see RLPAnalyzer#RLPAnalyzer(LanguageCode, String, EnumSet, String[])
      */
     public RLPJaAnalyzer(String rlpContextDef, EnumSet<PostType> postTypes, String allowedPOSTags[])
     {
         super(LanguageCode.JAPANESE, rlpContextDef, postTypes, allowedPOSTags);
     }

     /**
      * This constructor uses the part-of-speech filter with the default part-of-speech tag set.
      * @param rlpContextDef Context definition that RLP uses to process text: an XML string or path to XML file.
      * @param postTypes RLP Result types for which the tokenizer will generate tokens.
      * 
      * @see RLPAnalyzer#RLPAnalyzer(LanguageCode, String, EnumSet)
      */
     public RLPJaAnalyzer(String rlpContextDef, EnumSet<PostType> postTypes)
     {
         super(LanguageCode.JAPANESE, rlpContextDef, postTypes, DEFAULT_ALLOWED_POS_TAGS);
     }

     /**
      * This constructor uses default set of the post types, which are <ul>
      * <li>STEM (which is actually a lemma)</li>
      * <li>COMP (compound word decomposition)</li>
      * <li>POS (part-of-speech in Token's payload field)</li>
      * </ul>
      * Note that the many-to-one normalizer (used to be called JON) output and readings are not used.
      * @param rlpContextDef Context definition that RLP uses to process text: an XML string or path to XML file.
      * @see RLPAnalyzer#RLPAnalyzer(LanguageCode, String)
      */
     public RLPJaAnalyzer(String rlpContextDef)
     {
         super(LanguageCode.JAPANESE, rlpContextDef, DEFAULT_POST_TYPES, DEFAULT_ALLOWED_POS_TAGS);        
     }

     /**
      * This default constructor uses the default RLP Context which includes RCLU and JLA LPs.
      * RCLU is used for NKFC normalization and lower-casing of Latin alphabet.
      * JLA is used with the deep-decomposition option.
      * @see RLPAnalyzer#RLPAnalyzer(LanguageCode)
      */
     public RLPJaAnalyzer()
     {
         super(LanguageCode.JAPANESE, DEFAULT_CONTEXT_DEF, DEFAULT_POST_TYPES, DEFAULT_ALLOWED_POS_TAGS);
     }


     /**
      * (Internal use only)
      * Tokenizes a Japanese sentence, and displays the results.
      *
      * @param args A Japanese sentence (arg[0]). If you do not include an arg, a default sentence
      * is processed.
      */
     public static void main(String[] args) 
     {

         Analyzer anaDefault = new RLPJaAnalyzer();
         Analyzer anaNoLemmaCompElemOnly = new RLPJaAnalyzer(getDefaultContextDefinition(), EnumSet.of(PostType.COMP));
         Analyzer anaLemmaSynCompElemOnly = new RLPJaAnalyzer(getDefaultContextDefinition(), EnumSet.of(PostType.WORD, PostType.STEM, PostType.COMP, PostType.POS));
         Analyzer anaLemmaOnlyNoDecomp = new RLPJaAnalyzer(getDefaultContextDefinition(), EnumSet.of(PostType.STEM, PostType.COMP, PostType.POS));


         String text = "黒澤明は、昭和60年、映画界では初めて文化勲章を受賞した。さらにアメリカでｱｶﾃﾞﾐｰ名誉賞を受賞した。国会議事堂前で乗り換えの電車を待ちました。";
         if(args.length>=1)
         {
             text = args[0];
         }
         try 
         {
             System.out.println("Analyzing:"+text);
             AnalyzerUtils.displayTokensWithFullDetails(anaDefault, text);
             AnalyzerUtils.displayTokensWithFullDetails(anaNoLemmaCompElemOnly, text);
             AnalyzerUtils.displayTokensWithFullDetails(anaLemmaSynCompElemOnly, text);
             AnalyzerUtils.displayTokensWithFullDetails(anaLemmaOnlyNoDecomp, text);
         } 
         catch (IOException e) 
         {
             e.printStackTrace();
         }
     }

}
