////-*- mode: Java; coding: utf-8 -*-
/******************************************************************************
 ** This data and information is proprietary to, and a valuable trade secret
 ** of, Basis Technology Corp.  It is given in confidence by Basis Technology
 ** and may only be used as permitted under the license agreement under which
 ** it has been distributed, and in no other way.
 **
 ** Copyright (c) 2008 Basis Technology Corp. All rights reserved.
 **
 ** The technical data and information provided herein are provided with
 ** `limited rights', and the computer software provided herein is provided
 ** with `restricted rights' as those terms are defined in DAR and ASPR
 ** 7-104.9(a).
 ******************************************************************************/
package com.basistech.rlp.lucene;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.util.HashSet;

import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;

/**
 * This Filter removes tokens based on the POS (part-of-speech) tags that are stored
 * in each token's payload by RLPTokenizer.
 */
public class RLPPOSFilter extends TokenFilter {

    private HashSet<ByteBuffer> posSet;


    /**
     * Constructor that takes the allowed POS list as an array of Strings.
     * Only Tokens of the part-of-speech tags that are listed in the list will pass this filter.
     * Note that the part-of-speech tags depend on the language. 
     * Please refer the RLP Developer's Guide for the list of POS tags.
     * @param input The source of tokens to this filter.
     * @param posArray An array of the POS tags that will pass through this filter.
     */
    public RLPPOSFilter(TokenStream input, String[] posArray)
    {
        super(input);
        posSet = new HashSet<ByteBuffer>();
        for(int i=0; i<posArray.length; ++i)
            try {
                posSet.add(ByteBuffer.wrap(posArray[i].getBytes("UTF-8")));
            } catch (UnsupportedEncodingException e) {
                new RLPLuceneException("Error in converting POS String to byte[]", e);
            }
    }

    /**
     * Iterates to the next token until a token whose part-of-speech tag is in the list, 
     * or the tokens are exhausted.
     * @param result A token that will be used to return the next token.
     * @return The next token;  null if there are no more tokens.
     * @see org.apache.lucene.analysis.TokenStream#next(Token)
     */
    @Override
    public Token next(Token result) throws IOException {
        Token tok = input.next(result);
        if(tok==null) return null;
        try{
            while (!posSet.contains(ByteBuffer.wrap(tok.getPayload().getData()))) {
                tok = input.next(tok);
                if (tok == null)
                    return null;
            }
        }catch(NullPointerException npe){
            throw new RLPLuceneException("Payload is probably missing. Has RLPTokenizer been initialized so that it stores POS tags in Tokens? If using Solr, RLPTokenizerFilterFilter should appear without postPartOfSpeech=\"false\" (default: true) in schema.xml.");
        }
        return tok;
    }
}
