/******************************************************************************
** This data and information is proprietary to, and a valuable trade secret
** of, Basis Technology Corp.  It is given in confidence by Basis Technology
** and may only be used as permitted under the license agreement under which
** it has been distributed, and in no other way.
**
** Copyright (c) 2007-2008 Basis Technology Corp. All rights reserved.
**
** The technical data and information provided herein are provided with
** `limited rights', and the computer software provided herein is provided
** with `restricted rights' as those terms are defined in DAR and ASPR
** 7-104.9(a).
******************************************************************************/

package com.basistech.rlp.lucene;

import java.lang.reflect.*;
import java.io.IOException;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.nio.CharBuffer;
import java.nio.IntBuffer;
import java.util.EnumSet;
import java.util.Map;

import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.Tokenizer;
import org.apache.lucene.index.Payload;

import com.basistech.rlp.RLPConstants;
import com.basistech.rlp.RLPContext;
import com.basistech.rlp.RLPException;
import com.basistech.rlp.RLPResultRandomAccess;
import com.basistech.util.LanguageCode;

import com.basistech.rlp.extra.RLPEnvironmentHolder;

/**
 * A Tokenizer that uses RLP Basis Linguistic language processors, such as BL1, CLA, JLA, KLA, and ARBL.  
 * Objects of this class hold a reference to a com.basistech.rlp.RLPContext object, 
 * and reuse that context for each new input. These objects close their context when they are closed.
 * <br/>
 * <em>The API of this class has changed from that of the RLPTokenizer previuosly provided in the samples 
 * directory.</em>
 * If you have an application that was written to the sample integration code, you may need to modify your 
 * application code to use this package.
 * <br/>
 * This Tokenizer will convert various RLP result types into a TokenStream. 
 * setPost<i>Name</i>(boolean) methods of this class control which of the results of RLP are included 
 * in the token stream. 
 * Alternatively, {@link #setPostTypes} can be used to turn on a set of result types.
 * If you configure this class to include more than one RLP result type, 
 * this class will follow the Lucene convention for "synonym" tokens:
 * it will set the position increment to 0 for all but the first Token. 
 * <br/>
 * The RLP result type is stored in the Type field of each Lucene Token in a short name listed in the table below.
 * The table also list the (<i>Name</i> part of setPost<i>Name</i>(boolean) method that determins whether the 
 * result type should generate a token, and the corresponding getter method isPost<i>Name</i>()).
 * <table border="1">
 * <tr><th>RLP Result Type</th><th>setPost<i>Name</i><th>Short name</th></tr>
 * <tr><td>TOKEN</td><td>Word</td><td>word</td></tr>
 * <tr><td>NORMALIZED_TOKEN</td><td>Word</td><td>norm</td>
 * <tr><td>MANY_TO_ONE_NORMALIZED_TOKEN</td><td>M1NormalizedToken</td><td>m1norm</td>
 * <tr><td>STEM</td><td>Stem</td><td>stem</td></tr>
 * <tr><td>LEMMA</td><td>Lemma</td><td>lemma</td></tr>
 * <tr><td>COMPOUND</td><td>CompoundComponents</td><td>comp</td></tr>
 * <tr><td>READING</td><td>Readings</td><td>reading</td></tr>
 * <tr><td>ROOTS</td><td>Root</td><td>root</td></tr>
 * <tr><td>PART_OF_SPEECH</td><td>PartOfSpeech</td><td>(see below)</td></tr>
 * </table>
 * <br/>
 * <font size=-1>The method root names usually follow the names of the RLP result types, the symbolic values that can be found in {@link com.basistech.rlp.RLPConstants} except:
 * <ul>
 * <li>An undescore (_) is replaced by the capitalization of the following letter.</li>
 * <li>For brivity, "M1" is used for "many to one".</li>
 * <li>The name is in the plural form only if the input word can generate more than one result token.</li>
 * </ul>
 * </font>
 * <p>
 * For example, if a RLPTokenizer is created in this way:
 * <code><pre>
 * tok = RLPTokenizer(new StringReader("She said he said."), 
 *                    LanguageCode.ENGLISH, "/some/where/rlp-context-no-op.xml", 
 *                    EnumSet.of(PostType.WORD, PostType.STEM); 
 * </pre></code>
 * Successive calls to {@link #next()} will return these Tokens:
 * <ol>
 * <li>("She", "word", +1)</li>
 * <li>("she", "stem", +0)</li>
 * <li>("said", "word", +1)</li>
 * <li>("say", "stem", +0)</li>
 * <li>("he", "word", +1)</li>
 * <li>("he", "stem", +0)</li>
 * <li>("said", "word", +1)</li>
 * <li>("say", "stem", +0)</li>
 * <li>(".", "word", +1)</li>
 * <li>(".", "stem", +0)</li>
 * <li>null</li>
 * </ol>
 * where a token is represented by the triplet (text, type, position increment).
 * </p><p>
 * Note that Token pairs for "he" and "." are identical except for the position increment values.
 * To remove duplicate, you can apply Solr's {@link org.apache.solr.analysis.RemoveDuplicatesTokenFilter}.
 * (Specify its factory class, {@link org.apache.solr.analysis.RemoveDuplicatesTokenFilterFactory}, instead, 
 * in schema.xml, if you are using it from Solr.)
 * You can also apply Lucene's standard org.apache.lucene.analysis.LowerCaseFilter (LowerCaseFilterFactory) 
 * before RemoveDuplicatesTokenFilter to remove almost duplicate the "she" stem Token.
 * </p><p>
 * If you turn on the compound handling (by calling {@link #setPostCompoundComponents(boolean)} with true, 
 * or including PostType.COMP in the EnumSet given to the constructor), please note that:
 * <ul>
 * <li>Only compound words have these "comp" tokens.</li>
 * <li>Token corresponding to each component has the same start and end offsets of the original word.</li>
 * <li>Appending all the "comp" Tokens for the original word does not always yield the same word.
 * </ul>
 * Let's take the German compound word "Ferienwohnungen" (vacation apartments) as an example.
 * Assuming only the word and compound components are turned on, the RLPTokenizer will generate these Tokens:
 * <ol>
 * <li>("Ferienwohnungen", "word", +1, 0, 15)</li>
 * <li>("Ferien", "comp", +0, 0, 15)</li>
 * <li>("Wohnung", "comp", +0, 0, 15)</li>
 * </ol>
 * Notice that "wohnung" is said to start at the position 0 and end at 15, although it only has 7 letters and it actually starts at position 6. 
 * Also notice that the components are given in the singular form, although the original word was in the plural form, and each is capitalized.
 * </p><p>
 * Unlike other result types, {@link com.basistech.rlp.RLPConstants#PART_OF_SPEECH} does not generate its own Token.  
 * Instead, the part-of-speach tag will be attached as a Payload to each Token generated from the word.
 * <code> aToken.getPayload().getData() </code> will return its part-of-speach tag as a byte array of US-ASCII characters.
 * To have this as a String, call
 * <code> new String(aToken.getPayload().getData(), "US-ASCII") </code>.
 * </p><p>
 * This class does not handle the result type from the RLP Stopwords language processor.
 * Use {@link RLPPOSFilter} of this package, the standard Lucene StopFilter, 
 * or the builtin stopword feature of some language processors such as JLA.
 * </p>
 * <p>
 * When writing code that inspects the type field of Tokens, use the constant fields of this class such as {@link #TOKEN_TYPE_STRING_WORD} instead of the literal string "word", for example, so that misspellings can be caught by the compiler.
 * </p>
 */
public class RLPTokenizer extends Tokenizer {
   
    /**
     * This enum is used to specify a set of result types that this Tokenizer will use, in the form of EnumSet&lt;PostType&gt;.
     */
    public enum PostType {WORD, NORM, M1NORM, LEMMA, STEM, POS, COMP, READING, ROOT};
 
    // Names found in the type field of Lucene kokens.
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#TOKEN}.
     */
    public final static String TOKEN_TYPE_STRING_WORD = "word";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#NORMALIZED_TOKEN}.
     */
    public final static String TOKEN_TYPE_STRING_NORMALIZED_TOKEN = "norm";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#MANY_TO_ONE_NORMALIZED_TOKEN}.
     */
    public final static String TOKEN_TYPE_STRING_MANY_TO_ONE_NORMALIZED_TOKEN = "m1norm";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#STEM}.
     */
    public final static String TOKEN_TYPE_STRING_STEM = "stem";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#LEMMA}.
     */     
    public final static String TOKEN_TYPE_STRING_LEMMA = "lemma";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#COMPOUND}.
     */
    public final static String TOKEN_TYPE_STRING_COMPOUND_COMPONENT = "comp";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#READING}.
     */
    public final static String TOKEN_TYPE_STRING_READING = "reading";
    /**
     * The string that will be found in the type field of tokens that correspond to {@link RLPConstants#ROOTS}.
     */
    public final static String TOKEN_TYPE_STRING_ROOT = "root";
 
    // These two static variables, initialization block, and public method
    // are to make it possible for this module to work with two incompatible
    // versions of RLP: 6.0 and 6.5.
    private static boolean rlp_65_or_above = false; // Will be true if running under RLP 6.5 or later.
    private static int m1normtokconst = 0; // Will be RLPConstants.MANY_TO_ONE_NORMALIZED_TOKEN if running under RLP 6.5 or later.
    static{
            try {
                Field m1normtokenfld = RLPConstants.class.getField("MANY_TO_ONE_NORMALIZED_TOKEN");
                m1normtokconst = m1normtokenfld.getInt(null);
                rlp_65_or_above = true;
            } catch (NoSuchFieldException ex) {
                // This probably means the RLP is 6.0 or earlier.
            } catch (Exception ex){
                throw new RLPLuceneException("Error during static initialization.", ex);
            }
    }
     
    private static final Payload emptypl = new Payload(new byte[]{}); // Empty byte array payload, used as a dummy value when no POS is available.

    // Configuration properties
    private boolean postWord = true;
    private boolean postNormalizedToken;
    private boolean postM1NormalizedToken;
    private boolean postStem;
    private boolean postLemma;
    private boolean postPartOfSpeech;
    private boolean postCompoundComponents;
    private boolean postReadings;
    private boolean postRoot;
    
    private LanguageCode language = com.basistech.util.LanguageCode.UNKNOWN;
    private LanguageCode detectedLanguage = com.basistech.util.LanguageCode.UNKNOWN;
    
    private RLPContext context;
    private boolean processed;
    private Reader input;
    private int currentTokenIndex;
    private int tokenCount;
    
    private String[] tokensData;
    private String[] partsOfSpeechData;
    private String[] normalizedTokensData;
    private String[] m1NormalizedTokensData;
    private String[] stemsData;
    private String[] lemmasData;
    private Map<Integer, String[]> compoundComponentsData;
    private Map<Integer, String[]> readingsData;
    private String[] rootsData;
    private IntBuffer tokenOffsets;

    // Misc limits and tunable numbers.
    private final static int MAX_TOKENS_PER_WORD = 100; // Maximum number of Lucene Tokens that can be generated for each real token.
    private final static int READ_BUF_SIZE = 0x10000; // #characters to be read by one read() operation on the Reader.

    private String[] tokenDataQueue = new String[MAX_TOKENS_PER_WORD];
    private String[] tokenTypeQueue = new String[MAX_TOKENS_PER_WORD];
    private int currentStart;
    private int currentEnd;
    private int queueCount;
    private int queueIndex;
    private Payload queuePayload; // POS of the current word as byte[] in Payload/
    
    /**
     * This is the only constructor.  This class does not have the default constructor.
     * @param input The input reader.
     * @param lang Two-letter language code, supported by RLP.  Specify null to have RLI detect the language. 
     * (RLI feature must be licensed.)
     * @param rlpContextDef Path to RLP Context Defnition file, or the definition itself (must start with "&lt;").
     * @param postTypes Result types that should generate tokens.  If null is specified, only the "word" 
     * token will be generated unless setPost<i>Name</i>() is called <i>before</i> the first next() call.
     */
    public RLPTokenizer(Reader input, LanguageCode lang, String rlpContextDef, EnumSet<PostType> postTypes)
    {
        super(input);
        this.input = input; 
        this.language = lang;
        if(postTypes!=null) setPostTypes(postTypes);
        try {
            if(rlpContextDef.startsWith("<")){ // The config string itself
                context = RLPEnvironmentHolder.getEnvironment().getContextFromBuffer(rlpContextDef);
            }else{ // Assume this is a file path
                context = RLPEnvironmentHolder.getEnvironment().getContextFromFile(rlpContextDef);
            }
        } catch (Exception e) {
            throw new RLPLuceneException("Failed to obtain the RLPEnvironment object.", e);
        }
       
    }
    
    private RLPTokenizer(){ // Hide default constructor.
        throw new java.lang.UnsupportedOperationException();
    } 
    
    /** For internal use only.
     * @param input Input reader.
     * @see org.apache.lucene.analysis.Tokenizer#reset(java.io.Reader)
     */
    @Override
    public void reset(Reader input) throws IOException {
        super.reset(input);
        reset();
        try {
            process();
        } catch (RLPException e) {
            throw new RLPLuceneException(e);
        }
    }

    /** For internal use only.
     * @see org.apache.lucene.analysis.TokenStream#reset()
     */
    @Override
    public void reset() throws IOException {
        super.reset();
        currentTokenIndex = 0;
        queueCount = 0;
        queueIndex = 0;
    }

    /**
     * 
     * Returns the next token in the token stream.
     * Note: {@link #setLanguage(LanguageCode)} must be called before the first call to this method
     * unless RLI is in your context and included in your license.
     * @param result For best performance use new Token() to begin, then the token returned by the previous call to this method.
     * @return The next token (result), or null if at end of stream.
     * @see org.apache.lucene.analysis.TokenStream#next(org.apache.lucene.analysis.Token)
      */
    @Override
    public Token next(Token result) throws IOException {
        if (!processed) {
            try {
                process();
            } catch (RLPException e) {
                throw new RLPLuceneException(e);
            }
        }

        if (queueIndex >= queueCount) {
            do{// This loop is usually iterated only once because fillQueue() usually updates queueCount >= 1.
                if(currentTokenIndex >= tokenCount) return null;
                queuePayload = fillQueue();
            }while(queueCount==0); // For example, if only postCompoundCompounent==true and the token was not a compound word, this happens.

        }
        
        if (queueIndex == 0) {
            result.setPositionIncrement(1);
        } else {
            result.setPositionIncrement(0);
        }
        
        result.setEndOffset(currentEnd);
        result.setStartOffset(currentStart);
        char[] data = tokenDataQueue[queueIndex].toCharArray();
        char[] termBuffer = result.termBuffer();
        if (data.length > termBuffer.length) {
            termBuffer = result.resizeTermBuffer(data.length);
        }
        System.arraycopy(data, 0, termBuffer, 0, data.length);
        result.setTermLength(data.length);
        result.setType(tokenTypeQueue[queueIndex]);
        
        // Add part-of-speach as a Payload to each Token.
        if (queuePayload!=null) {
           result.setPayload(queuePayload);
        }
        
        queueIndex ++;
        return result;
    }

    /*
     * For the current word, fill up the pair of queues with various types of text
     * corresponding to the word. 
     * @return Payload that contains the POS tag as ASCII string. Empty array payload, if POS was not returned from RLP when POS is requested.  null if POS is not requested.
     */
    private Payload fillQueue() {
        queueIndex = 0;
        queueCount = 0;
        currentStart = tokenOffsets.get(currentTokenIndex * 2);
        currentEnd = tokenOffsets.get((currentTokenIndex * 2) + 1);
        
        if(postWord) {
            tokenDataQueue[queueCount] = tokensData[currentTokenIndex];
            tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_WORD;
            queueCount ++;
        }
        if (postStem && stemsData != null && stemsData[currentTokenIndex] != null) {
            String stem = stemsData[currentTokenIndex];
            // Fix up bug RLP-2062 that leaves holes for words that are already in the dictionary form.
            if( stem.equals("")){
                stem = tokensData[currentTokenIndex];
            }
            tokenDataQueue[queueCount] = stem;
            tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_STEM;
            queueCount ++;
        }
        
        if (postLemma && lemmasData != null && lemmasData[currentTokenIndex] != null) {
            tokenDataQueue[queueCount] = lemmasData[currentTokenIndex];
            tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_LEMMA;
            queueCount ++;
        }

        
        if(postNormalizedToken && normalizedTokensData != null && normalizedTokensData[currentTokenIndex] != null){
            tokenDataQueue[queueCount] = normalizedTokensData[currentTokenIndex];
            tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_NORMALIZED_TOKEN;
            queueCount ++;
        }
        
        if(postM1NormalizedToken && m1NormalizedTokensData != null && m1NormalizedTokensData[currentTokenIndex] != null){
            tokenDataQueue[queueCount] = m1NormalizedTokensData[currentTokenIndex];
            tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_MANY_TO_ONE_NORMALIZED_TOKEN;
            queueCount ++;            
        }
        
        if (postRoot && rootsData != null && rootsData[currentTokenIndex] != null) {
            tokenDataQueue[queueCount] = rootsData[currentTokenIndex];
            tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_ROOT;
            queueCount ++;
        }
                
        if (postCompoundComponents && compoundComponentsData != null) {
            String[] comps = compoundComponentsData.get(new Integer(currentTokenIndex));
            if (comps != null) {
                for (String comp : comps) {
                    tokenDataQueue[queueCount] = comp;
                    tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_COMPOUND_COMPONENT;
                    queueCount ++;
                }
            }
        }
        
        if (postReadings && readingsData != null) {
            String[] reads = readingsData.get(new Integer(currentTokenIndex));
            if (reads != null) {
                for (String read : reads) {
                    tokenDataQueue[queueCount] = read;
                    tokenTypeQueue[queueCount] = TOKEN_TYPE_STRING_READING;
                    queueCount ++;
                }
            }
        }
        
        Payload pl = null;
        if(postPartOfSpeech){// Should put something in the payload.
            String pos;
            if (partsOfSpeechData != null && (pos = partsOfSpeechData[currentTokenIndex]) != null) {
                try {
                    pl = new Payload(pos.getBytes("US-ASCII"));
                } catch (UnsupportedEncodingException e) {
                    throw new RLPLuceneException(e);
                }
            }else{ // Will retrun byte[] of 0 element, rather than null.
                pl = emptypl;
            }
        }        
            
        currentTokenIndex++;        
        return pl;

    }


    @SuppressWarnings("unchecked")
    private void process() throws IOException, RLPException {
        // we need to convert the reader to a String. Some day, we'd want something more stream-like.
        // or a buffer, or something. A String works and is convenient.
        StringBuilder readInto = new StringBuilder();
        CharBuffer buffer = CharBuffer.allocate(READ_BUF_SIZE);
        while((input.read(buffer)) > 0) {
            buffer.flip();
            readInto.append(buffer);
            buffer.rewind();
        }
        String data = readInto.toString();
        int offset = 0;
        
        // the offset skips the magic language code if it is present.
        context.process(data.toCharArray(), offset, data.length(), language);
        RLPResultRandomAccess rrra = new RLPResultRandomAccess(context);
        detectedLanguage = rrra.getDetectedLanguage();
        
        // Always get tokensData even when setPostWord(false) is called.
        // This is so that we can get the token count at least.
        tokensData = (String[])rrra.getResultData(RLPConstants.TOKEN);
        tokenCount = tokensData.length;
        if(tokenCount == 0) return; //Don't waste time.
        
        tokenOffsets = (IntBuffer)rrra.getResultData(RLPConstants.TOKEN_OFFSET);

        if (postNormalizedToken) {            
            normalizedTokensData = (String[])rrra.getResultData(RLPConstants.NORMALIZED_TOKEN);
        }
        
        if (postM1NormalizedToken){
            // Transional measure.
            if(rlp_65_or_above){
                m1NormalizedTokensData = 
                        (String[])rrra.getResultData(m1normtokconst);
            }else{
               // JON LP posts the regular NORMALIZED_TOKEN notice.
               m1NormalizedTokensData = normalizedTokensData; 
            }
        }
        
        if (postStem) {
            stemsData = (String[])rrra.getResultData(RLPConstants.STEM);
        }
        
        if (postLemma) {
            lemmasData = (String[])rrra.getResultData(RLPConstants.LEMMA);
        }
        
        if (postRoot) {
            rootsData = (String[])rrra.getResultData(RLPConstants.ROOTS);
        }
        
        if (postPartOfSpeech) {
            partsOfSpeechData = (String[])rrra.getResultData(RLPConstants.PART_OF_SPEECH);
        }
        
        if (postCompoundComponents) {
            compoundComponentsData = (Map<Integer, String[]>)rrra.getResultData(RLPConstants.COMPOUND);
        }
        
        if (postReadings) {
            readingsData = (Map<Integer, String[]>)rrra.getResultData(RLPConstants.READING);
        }
        tokenCount = tokensData.length;
        processed = true;
    }
    
    /** For internal use.
     * @see org.apache.lucene.analysis.Tokenizer#close()
     */
    @Override
    public void close() throws IOException {
        super.close();
        context.close();
    }

    /**
     * Determines whether a token is being generated for each word in its original form in the text
     * @return true if a token is generated for each of the original words in the text.
     */
    public boolean isPostWord() {
        return postWord;
    }

    /**
     * Specifies whether to generate a token for each of the original words in the text.
     * @param b Pass true if a token is generatedfor each of the original words in the text. (Initial value: true)
     */
    public void setPostWord(boolean b) {
        this.postWord = b;
    }

    /**
     * Determines whether a token is generated for each normalized word (Arabic only).
     * @return true if a token is generatedfor each normalized word.
     */
    public boolean isPostNormalizedToken() {
        return postNormalizedToken;
    }
    
    /**
     * Specifies whether a token is generated for each normalized word (Arabic only).
     * @param b Pass true if a token is generatedfor each normalized word.
     */
    public void setPostNormalizedToken(boolean b) {
        this.postNormalizedToken = b;
    }
    
    /**
     * Determines whether a token is generated for each many-to-one normalized word (Japanese, and other languages).
     * @return true if a token is generated for each many-to-one normalized word.
     */
    public boolean isPostM1NormalizedToken() {
        return postM1NormalizedToken;
    }
    
    /**
     * Specifies whether a token is generated for each many-to-one normalized word (Japanese, and other languages).
     * @param b Pass true if a token is generated for each many-to-one normalized word.
     */
    public void setPostM1NormalizedToken(boolean b) {
        this.postM1NormalizedToken = b;
    }
    
    /**
     * Determines whether a token is generated for each stem.
     * @return true if a token should be generated for each stem.
     */
    public boolean isPostStem() {
        return postStem;
    }

    /**
     * Specifies whether a token is generated for each stem.
     * @param b Pass true if a token should be generated for each stem.
     */
    public void setPostStem(boolean b) {
        this.postStem = b;
    }

    /**
     * Determines whether a token is generated for each lemma (Arabic only).
     * @return true if a token should be generated for each lemma.
     */
    public boolean isPostLemma() {
        return postLemma;
    }

    /**
     * Specifies whether a token is generated for each lemma (Arabic only).
     * @param b Pass true if a token should be generated for each lemma.
     */
    public void setPostLemma(boolean b) {
        this.postLemma = b;
    }

    /**
     * Determines whether a part-of-speech (POS) is stored in each Token's Payload field.
     * @return true if a POS tag is stored in each Token's Payload field. 
     */
    public boolean isPostPartOfSpeech() {
        return postPartOfSpeech;
    }

    /**
     * Specifies whether a part-of-speech (POS) is stored in each Token's Payload field.
     * @param b Pass true if a POS tag is stored in each Token's Payload field. 
     */
    public void setPostPartOfSpeech(boolean b) {
        this.postPartOfSpeech = b;
    }

    /**
     * Determines whether component tokens are generated for each compound word (German, Dutch, Hungarian,
     * Chinese, Japanese, and Korean).
     * @return true if componet Tokens are generated for each compound word.
     */
    public boolean isPostCompoundComponents() {
        return postCompoundComponents;
    }

    /**
     * Specifies whether component tokens are generated for each compound word.
     * @param b Pass true if componet Tokens are generated for each compound word German, Dutch, Hungarian,
     * Chinese, Japanese, and Korean).
     */
    public void setPostCompoundComponents(boolean b) {
        this.postCompoundComponents = b;
    }

    /**
     * Determines whether reading Tokens are generated for each word that the language analyzer can predict
     * (Chinese and Japanese).
     * @return true if reading Tokens are generated.
     */
    public boolean isPostReadings() {
        return postReadings;
    }

    /**
     * Specifies whether reading Tokens are generated for each word that the language analyzer can predict
     * (Chinese and Japanese).
     * @param b Pass true if reading Tokens should be generated.
     */
    public void setPostReadings(boolean b) {
        this.postReadings = b;
    }

    /**
     * Determines whether a token is generated for each root (Arabic only).
     * @return true if a token is generated for each root.
     */
    public boolean isPostRoot(){
        return postRoot;
    }
    
    /**
     * Specifies whether a token is generated for each root (Arabic only).
     * @param b Pass true if a token is generated for each root.
     */
    public void setPostRoot(boolean b){
        this.postRoot = b;
    }

    /**
     * Bulk getter of types used to generate tokens.
     * @return set of enabled token-generating features.
     */    
    public EnumSet<PostType> getPostTypes(){
        EnumSet<PostType> es = EnumSet.noneOf(PostType.class);
        if(isPostWord()) es.add(PostType.WORD);
        if(isPostNormalizedToken()) es.add(PostType.NORM);
        if(isPostM1NormalizedToken()) es.add(PostType.M1NORM);
        if(isPostLemma()) es.add(PostType.LEMMA);
        if(isPostStem()) es.add(PostType.STEM);
        if(isPostPartOfSpeech()) es.add(PostType.POS);
        if(isPostCompoundComponents()) es.add(PostType.COMP);
        if(isPostReadings()) es.add(PostType.READING);
        if(isPostRoot()) es.add(PostType.ROOT);  
        
        return es;
    }

    /**
     * Bulk setter of token-generating features.
     * Calling this is equivalent of calling setPost<i>Name</i>(true or false) for all features.
     * You must use EnumSet that includes PostType.POS in order to use RLPPOSFilter.
     * @param es EnumSet of PostType values, e.g. EnumSet.of(PostType.WORD, PostType.STEM)
     */
    public void setPostTypes(EnumSet<PostType> es){
        setPostWord(es.contains(PostType.WORD));
        setPostNormalizedToken(es.contains(PostType.NORM));
        setPostM1NormalizedToken(es.contains(PostType.M1NORM));
        setPostLemma(es.contains(PostType.LEMMA));
        setPostStem(es.contains(PostType.STEM));
        setPostPartOfSpeech(es.contains(PostType.POS));
        setPostCompoundComponents(es.contains(PostType.COMP));
        setPostReadings(es.contains(PostType.READING));
        setPostRoot(es.contains(PostType.ROOT));       
    }
    
    
    /**
     * Returns the language previously set by {@link #setLanguage(LanguageCode)}.
     * @return The language code.  
     * Note this method does <em>not</em> return the detected language by RLI LP.
     * Use {@link #getDetectedLanguage()} for that purpose.
     */
    public LanguageCode getLanguage() {
        return language;
    }

    /**
     * Set the language of the text.  
     * <br/>
     * Note: In order to convert the two-letter language code such as "en" (for English) 
     * to a {@link com.basistech.util.LanguageCode},  
     * use {@link com.basistech.util.LanguageCode#lookupByISO639(String)}.
     *<br/>
     * To have RLI (Rosette Language Identifier) detect the language automatically, 
     * include RLI LP in your RLP Context Definition, 
     * and specify {@link com.basistech.util.LanguageCode#UNKNOWN},
     * or do not call this method at all.
     * RLI is licensed separately.
     * 
     * @param language The language of the text. 
     */
    public void setLanguage(LanguageCode language) {
        this.language = language;
    }

    /**
     * Return the detected language by RLI, if it is enabled.  
     * Otherwise, it just returns the language set by previous call to {@link #setLanguage(LanguageCode)}.
     * @return The detected language.
     */
    public LanguageCode getDetectedLanguage(){
        return detectedLanguage;
    }
}
