package com.basistech.rlp.solr;

import java.io.Reader;
import java.util.Map;

import org.apache.lucene.analysis.TokenStream;
import org.apache.solr.analysis.BaseTokenizerFactory;
import org.apache.solr.common.ResourceLoader;
import org.apache.solr.common.SolrException;
import org.apache.solr.util.plugin.ResourceLoaderAware;

import com.basistech.rlp.lucene.RLPTokenizer;
import com.basistech.util.LanguageCode;


/**
 * Solr tokenizer factory for RLPTokenizer. 
 * This class name should be specified in the &lt;tokenizer&gt; element in schema.xml.
 * Following attributes will be recognized:
 * <ul>
 * <li>"rlpContext"  Path to the RLP Context Definition XML file. 
 * The path can be an absolute path or a relative path to the current working directory.
 * (This cannot be relative to the solr/conf directory unlike other filters.)
 * Optional but strongly recommended to specify the context definition that is optimized for your set of languages.</li>
 * <li>"lang"  The two letter ISO language code. This is mandatory unless RLI is used to detect the language.</li>
 * <li>"postWord"  Set false to exclude the raw word (after RCLU transformation, if any). Default: true</li>
 * <li>"postNormalizedToken"  Set true to have (Arabic) normalized tokens. Default: false </li>
 * <li>"postM1NormalizedToken" Set true to have word-variation (many-to-one) normalized tokens. Default: false</li>
 * <li>"postStem" Set true to have STEM tokens. Default: false</li>
 * <li>"postLemma"  Set true to have LEMMA tokens. Default: false</li>
 * <li>"postPartOfSpeech" Set true to have a part-of-speech tag embeded in each Token's payload field. Default: true</li>
 * <li>"postCompoundComponents" Set true to have tokens, one for each subcomponent of a compound word. Default: false </li>
 * <li>"postReadings" Set true to have Tokens each corresponding to a possible reading Default: false.</li>
 * <li>"postRoot" Set true to have (Arabic) root tokens. Default: false</li>
 * </ul>
 * <br/>
 * Note that post<i>Name</i> has an effect only if the underlying base linguistic processor supports the feature.
 * For example, postCompoundComponents="true" will have a real effect only 
 * for Japanese (lang="ja"), German (de), Dutch (nl) or Hungarian (hu).
 * For almost the similar effect of the dictionary form of the words, 
 * use postLemma="true" for Arabic and postStem="true" for other languages.
 * <br/>
 * Most public set<i>Property</i> and is<i>Property</i> or get<i>Property</i> methods are there in anticipation of Solr's eventual adoption of Spring.
 */
public class RLPTokenizerFactory extends BaseTokenizerFactory implements ResourceLoaderAware {
    
    // Here we have a clone of all the configuration properties of the tokenizer itself.
    private boolean postWord = true;
    private boolean postNormalizedToken;
    private boolean postM1NormalizedToken;
    private boolean postLemma;
    private boolean postStem;
    private boolean postPartOfSpeech = true;
    private boolean postCompoundComponents;
    private boolean postReadings;
    private boolean postRoot;
    private LanguageCode language = LanguageCode.UNKNOWN;
    private String rlpContextDef = com.basistech.rlp.lucene.RLPAnalyzer.getDefaultContextDefinition();
    private RLPTokenizer tokenizer;
    
    /**
     * Creates a token stream using {@link RLPTokenizer}.
     * @param input The input reader.
     * @return The token stream.
     */
    public TokenStream create(Reader input) {

        tokenizer = new RLPTokenizer(input, language, rlpContextDef, null);
        tokenizer.setPostWord(postWord);
        tokenizer.setPostNormalizedToken(postNormalizedToken);
        tokenizer.setPostM1NormalizedToken(postM1NormalizedToken);
        tokenizer.setPostLemma(postLemma);        
        tokenizer.setPostStem(postStem);
        tokenizer.setPostPartOfSpeech(postPartOfSpeech);
        tokenizer.setPostCompoundComponents(postCompoundComponents);                
        tokenizer.setPostReadings(postReadings);
        tokenizer.setPostRoot(postRoot);
        return tokenizer;
    }


    /**
     * Retrieves the language code that has been set (LanguageCode.UNKNOWN if it hasn't been set).
     * @return The language that has been set.
     */
    public LanguageCode getLanguage() {
        return language;
    }

    /**
     * Designates the language of the text to be processed.
     * @param language The language of the text.
     *                
     */
    public void setLanguage(LanguageCode language) {
        this.language = language;
    }

    /**
     * Retrieves the path to the RLP XML context definition file or string.
     * @return The RLP context definition.
     */
    public String getRLPContextDef() {
        return rlpContextDef;
    }

    /**
     * Sets the path to the RLP XML context definition file or string must start with "&lt;").
     * @param rlpContextDef Path to RLP context definition file or string.
     */
    public void setRLPContextDef(String rlpContextDef) {
        this.rlpContextDef = rlpContextDef;
    }


    /**
     * Determines whether a token is being generated for each word in its original form in the text
     * @return true if a token is generated for each of the original words in the text.
     */
    public boolean isPostWord()
    {
        return postWord;
    }

    /**
     * Specifies whether to generate a token for each of the original words in the text.
     * @param b Pass true if a token is generatedfor each of the original words in the text. (Initial value: true)
     */
    public void setPostWord(boolean b)
    {
        this.postWord = b;
    }

    /**
     * Determines whether a token is generated for each normalized word (Arabic only).
     * @return true if a token is generatedfor each normalized word.
     */
    public boolean isPostNormalizedToken()
    {
        return postNormalizedToken;
    }

    /**
     * Specifies whether a token is generated for each normalized word (Arabic only).
     * @param b Pass true if a token is generatedfor each normalized word.
     */
    public void setPostNormalizedToken(boolean b)
    {
        this.postNormalizedToken = b;
    }

    /**
     * Determines whether a token is generated for each many-to-one normalized word (Japanese, and other languages).
     * @return true if a token is generated for each many-to-one normalized word.
     */
    public boolean isPostM1NormalizedToken()
    {
        return postM1NormalizedToken;
    }

    /**
     * Specifies whether a token is generated for each many-to-one normalized word (Japanese, and other languages).
     * @param b Pass true if a token is generated for each many-to-one normalized word.
     */
    public void setPostM1NormalizedToken(boolean b)
    {
        this.postM1NormalizedToken = b;
    }

    /**
     * Determines whether a token is generated for each stem.
     * @return true if a token should be generated for each stem.
     */
    public boolean isPostStem()
    {
        return postStem;
    }

    /**
     * Specifies whether a token is generated for each stem.
     * @param b Pass true if a token should be generated for each stem.
     */
    public void setPostStem(boolean b)
    {
        this.postStem = b;
    }

    /**
     * Determines whether a token is generated for each lemma (Arabic only).
     * @return true if a token should be generated for each lemma.
     */
    public boolean isPostLemma()
    {
        return postLemma;
    }

    /**
     * Specifies whether a token is generated for each lemma (Arabic only).
     * @param b Pass true if a token should be generated for each lemma.
     */
    public void setPostLemma(boolean b)
    {
        this.postLemma = b;
    }

    /**
     * Determines whether a token is generated for each root (Arabic only).
     * @return true if a token is generated for each root.
     */
    public boolean isPostRoot()
    {
        return postRoot;
    }

    /**
     * Specifies whether a token is generated for each root (Arabic only).
     * @param b Pass true if a token is generated for each root.
     */
    public void setPostRoot(boolean b)
    {
        this.postRoot = b;
    }

    /**
     * Determines whether a part-of-speech (POS) is stored in each Token's Payload field.
     * @return true if a POS tag is stored in each Token's Payload field. 
     */
    public boolean isPostPartOfSpeech()
    {
        return postPartOfSpeech;
    }

    /**
     * Specifies whether a part-of-speech (POS) is stored in each Token's Payload field.
     * @param b Pass true if a POS tag is stored in each Token's Payload field. 
     */
    public void setPostPartOfSpeech(boolean b)
    {
        this.postPartOfSpeech = b;
    }

    /**
     * Determines whether component tokens are generated for each compound word (German, Dutch, Hungarian,
     * Chinese, Japanese, and Korean).
     * @return true if componet Tokens are generated for each compound word.
     */
    public boolean isPostCompoundComponents()
    {
        return postCompoundComponents;
    }

    /**
     * Specifies whether component tokens are generated for each compound word.
     * @param b Pass true if componet Tokens are generated for each compound word German, Dutch, Hungarian,
     * Chinese, Japanese, and Korean).
     */
    public void setPostCompoundComponents(boolean b)
    {
        this.postCompoundComponents = b;
    }

    /**
     * Determines whether reading Tokens are generated for each word that the language analyzer can predict
     * (Chinese and Japanese).
     * @return true if reading Tokens are generated.
     */
    public boolean isPostReadings()
    {
        return postReadings;
    }

    /**
     * Specifies whether reading Tokens are generated for each word that the language analyzer can predict
     * (Chinese and Japanese).
     * @param b Pass true if reading Tokens should be generated.
     */
    public void setPostReadings(boolean b)
    {
        this.postReadings = b;
    }

    /**
     * An implementation of {@link ResourceLoaderAware#inform(ResourceLoader)} as required by the interface.
     * @param loader See {@link ResourceLoaderAware#inform(ResourceLoader)}.
     */
    public void inform(ResourceLoader loader) {
        ;
        for(Map.Entry<String, String> me: args.entrySet()){
            String key = me.getKey();
            String val = me.getValue();
            if(key.equals("rlpContext")){
                rlpContextDef = val;
            }else if(key.equals("lang") && !val.equals("")){
                language = LanguageCode.lookupByISO639(val.toLowerCase());
            }else if(key.equals("postWord")){
                postWord = Boolean.parseBoolean(val);
            }else if(key.equals("postNormalizedToken")){
                postNormalizedToken = Boolean.parseBoolean(val);
            }else if(key.equals("postM1NormalizedToken")){
                postM1NormalizedToken = Boolean.parseBoolean(val);
            }else if(key.equals("postLemma")){
                postLemma = Boolean.parseBoolean(val);;
            }else if(key.equals("postStem")){
                postStem = Boolean.parseBoolean(val);;
            }else if(key.equals("postPartOfSpeech")){
                postPartOfSpeech = Boolean.parseBoolean(val);;
            }else if(key.equals("postCompoundComponents")){
                postCompoundComponents = Boolean.parseBoolean(val);;
            }else if(key.equals("postReadings")){
                postReadings = Boolean.parseBoolean(val);;     
            }else if(key.equals("postRoot")){
                postRoot = Boolean.parseBoolean(val);;
            }else{
                throw new RLPSolrException(
                        SolrException.ErrorCode.SERVER_ERROR,
                        "Configuration Error: unknown attribute '" + key + "'");                
            }
        }
    }

}
